# -*- coding: utf-8 -*-

"""
***************************************************************************
    OTBUtils.py
    ---------------------
    Date                 : August 2012
    Copyright            : (C) 2012 by Victor Olaya
                           (C) 2013 by CS Systemes d'information (CS SI)
    Email                : volayaf at gmail dot com
                           otb at c-s dot fr (CS SI)
    Contributors         : Victor Olaya
                           Julien Malik, Oscar Picas  (CS SI) - add functions to manage xml tree
                           Alexia Mondot (CS SI) - add a trick for OTBApplication SplitImages
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""
from builtins import str

__author__ = 'Victor Olaya'
__date__ = 'August 2012'
__copyright__ = '(C) 2012, Victor Olaya'
# This will get replaced with a git SHA1 when you do a git archive
__revision__ = '$Format:%H$'

import os
import re
import subprocess

from processing.core.ProcessingConfig import ProcessingConfig
from processing.core.ProcessingLog import ProcessingLog
from processing.tools.system import isMac, isWindows

from qgis.core import (Qgis, QgsMessageLog)
from qgis.PyQt.QtCore import QCoreApplication

ACTIVATE = "OTB_ACTIVATE"
FOLDER = "OTB_FOLDER"
APP_FOLDER = "OTB_APP_FOLDER"
VERSION = "OTB_VERSION"
SRTM_FOLDER = "OTB_SRTM_FOLDER"
GEOID_FILE = "OTB_GEOID_FILE"
DOWNLOAD_URL = "https://orfeo-toolbox.org/packages/archives/OTB/"
DEFAULT_RAM_VALUE = 128
OTB_CLI_FILE =  ''

import re
def version():
    return ProcessingConfig.getSetting(VERSION) or '0.0.0'

def otbFolder():
    if ProcessingConfig.getSetting(FOLDER):
        return os.path.normpath(os.sep.join(re.split(r'\\|/', ProcessingConfig.getSetting(FOLDER))))
    else:
        return None

def appFolder():
    app_folder = ProcessingConfig.getSetting(APP_FOLDER)
    if app_folder:
      return os.pathsep.join(app_folder.split(';'))
    else:
      return None

def srtmFolder():
    return ProcessingConfig.getSetting(SRTM_FOLDER) or ''

def geoidFile():
    return ProcessingConfig.getSetting(GEOID_FILE) or ''

def executeOtb(commands, feedback, addToLog=True):
    loglines = []
    with subprocess.Popen(
            commands,
            shell=True,
            stdout=subprocess.PIPE,
            stdin=subprocess.DEVNULL,
            stderr=subprocess.STDOUT,
            universal_newlines=True
            ) as proc:
        try:
            for line in iter(proc.stdout.readline, ''):
                line = line.strip()
                #'* ]' and '  ]' says its some progress update
                #print('line[-3:]',line[-3:])
                if line[-3:] == "* ]" or line[-3:] == "  ]":
                    part = line.split(':')[1]
                    percent = part.split('%')[0]
                    try:
                        if int(percent) >= 100:
                            loglines.append(line)
                        feedback.setProgress(int(percent))
                    except:
                        pass
                else:
                    loglines.append(line)
        except BaseException as e:
            loglines.append(str(e))
            pass

        for logline in loglines:
            if feedback is None:
                QgsMessageLog.logMessage(logline, 'Processing', Qgis.Info)
            else:
                feedback.pushConsoleInfo(logline)

        # for logline in loglines:
        #     if 'INFO' in logline or 'FATAL' in logline:
        #         if feedback is None:
        #             QgsMessageLog.logMessage(logline, 'Processing', Qgis.Info)
        #         else:
        #             feedback.pushConsoleInfo(logline)

def tr(string, context=''):
    if context == '':
        context = 'OTBUtils'
    return QCoreApplication.translate(context, string)
